/***************************************************************************
 *                                                                         *
 *                               RasTop 1.3.1                              *
 *                 Molecular Graphics Visualisation Tool                   *
 *                          by Philippe Valadon                            *
 *                              21 Mars 2001                               *
 *                                                                         *
 *                                *******                                  *
 *                                                                         *
 *                   Based on RasMol 2.6 by Roger Sayle                    *
 * Biomolecular Structures Group, Glaxo Wellcome Research & Development,   *
 *                      Stevenage, Hertfordshire, UK                       *
 *         Version 2.6, August 1995, Version 2.6.4, December 1998          *
 *                   Copyright (C) Roger Sayle 1992-1999                   *
 *                                                                         *
 *                  and Based on Mods by Arne Mueller                      *
 *                      Version 2.6x1, May 1998                            *
 *                   Copyright (C) Arne Mueller 1998                       *
 *                                                                         *
 *    and Based on Version 2.7.0, 2.7.1 Mods by Herbert J. Bernstein       *
 *           Bernstein + Sons, P.O. Box 177, Bellport, NY, USA             *
 *                      yaya@bernstein-plus-sons.com                       *
 *                    2.7.0 March 1999, 2.7.1 June 1999                    *
 *              Copyright (C) Herbert J. Bernstein 1998-1999               *
 *                                                                         *
 *                RasTop Version 1.3.x by Philippe Valadon                 *
 *                 P.O. Box 13014, La Jolla, CA 92039, USA,                *
 *                         valadon@ix.netcom.com                           *
 *                Copyright (C) Philippe Valadon 2000-2001                 *
 *                                                                         *
 *                                *******                                  *
 *                                                                         *
 * As written by Herbert Bernstein in Rasmol Version 2.7.1:                *
 *                                                                         *
 * Please read the file NOTICE for important notices which apply to this   *
 * package. If you are not going to make changes to Rasmol, you are not    *
 * only permitted to freely make copies and distribute them, you are       *
 * encouraged to do so, provided you do the following:                     *
 *   * 1. Either include the complete documentation, especially the file   *
 *     NOTICE, with what you distribute or provide a clear indication      *
 *     where people can get a copy of the documentation; and               *
 *   * 2. Please give credit where credit is due citing the version and    *
 *     original authors properly; and                                      *
 *   * 3. Please do not give anyone the impression that the original       *
 *     authors are providing a warranty of any kind.                       *
 *                                                                         *
 * If you would like to use major pieces of Rasmol in some other program,  *
 * make modifications to Rasmol, or in some other way make what a lawyer   *
 * would call a "derived work", you are not only permitted to do so, you   *
 * are encouraged to do so. In addition to the things we discussed above,  *
 * please do the following:                                                *
 *   * 4. Please explain in your documentation how what you did differs    *
 *     from this version of Rasmol; and                                    *
 *   * 5. Please make your modified source code available.                 *
 *                                                                         *
 * This version of Rasmol is not in the public domain, but it is given     *
 * freely to the community in the hopes of advancing science. If you make  *
 * changes, please make them in a responsible manner, and please offer us  *
 * the opportunity to include those changes in future versions of RasTop.  *
 ***************************************************************************/

/////////////////////////////////////////////////////////////////////////////
// mainfrm.cpp
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#include "RasTop.h"
#include "rasdoc.h"
#include "rasview.h"
#include "mainfrm.h"
#include "dialog.h"
#include "graphics.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////
// HTML Help

void InitHelpHTML( void )
{
	GETFRAME
	if( pFrame->m_HelpLine.IsEmpty() )
	{	WriteString("Warning: No indication where to find help!\n");
		WriteString("...      Check preferences!\n");
	} else
	{	::ShellExecute(NULL, NULL, pFrame->m_HelpLine, 
			NULL, _T("C:\\"),SW_SHOWNORMAL);
	}
}


/////////////////////////////////////////////////////////////////////////////
//Writing routines to infoline

void WriteChar(int ch)
{	GETFRAME
	static CString templine;
	static char Buffer[256];
	static char *ptr = Buffer;
	static int i = 0;

	if( pFrame->InfoSilentFlag!=false )
	{	if( pFrame->InfoSilentFlag==-1 )
		{	if( i==0 )
			{	ptr = templine.GetBuffer( 32384 );
				*ptr++ = ch;
				i++;
			} else if( ch == EOF )
			{	*ptr = '\0';
				templine.ReleaseBuffer();
				pFrame->m_szCommandText += templine + '\n';
				i = 0;
			} else if( i == 32000 )
			{	*ptr = '\0';
				templine.ReleaseBuffer();
				pFrame->m_szCommandText += "  " + templine;
				i = 0;
				ptr = templine.GetBuffer( 32384 );
			} else
			{	*ptr++ = ch;
				i++;
			}
		} 
		return;
	}

	if ( ch<32 || ch>126 )
		if( ch!=0x07 && ch!=0x0D && ch!=0x0A )
			return;

    switch( ch )
    {	case(0x07): MessageBeep(0);
				 	break;		      		      
		case(0x0D): 
		case(0x0A):	if( ptr==Buffer && FileDepth!=-1 )
						*ptr++ = 0x20;	//space to create a \n on echo command
					*ptr = '\0';
					templine = Buffer;					
					pFrame->m_wndCommandBar.OnAddInfoline(templine);
					ptr = Buffer;
					break;  
		default:	*ptr++ = ch;
    } 
} 

void WriteString( char *ptr )
{	while( *ptr )
		WriteChar( *ptr++ );
}

/////////////////////////////////////////////////////////////////////////////
// Helpers for saving/restoring window state and preferences
// With thanks to microsoft

static int newpref = 0;

static TCHAR BASED_CODE szSection[] = _T("Settings");
static TCHAR BASED_CODE szWindowPos[] = _T("WindowPos");
static TCHAR szFormat[] = _T("%u,%u,%d,%d,%d,%d,%d,%d,%d,%d");

static TCHAR BASED_CODE szPreferences[] = _T("Preferences");
static TCHAR szPrefReadFormat[] = _T("%d,%d,%d,%d,%d,%[^'\0']");
static TCHAR szPrefWriteFormat[] = _T("%d,%d,%d,%d,%d,%s");

static BOOL PASCAL NEAR ReadWindowPlacement(LPWINDOWPLACEMENT pwp)
{
	CString strBuffer = AfxGetApp()->GetProfileString(szSection, szWindowPos);
	if (strBuffer.IsEmpty())
		return false;

	WINDOWPLACEMENT wp;
	int nRead = _stscanf(strBuffer, szFormat,
		&wp.flags, &wp.showCmd,
		&wp.ptMinPosition.x, &wp.ptMinPosition.y,
		&wp.ptMaxPosition.x, &wp.ptMaxPosition.y,
		&wp.rcNormalPosition.left, &wp.rcNormalPosition.top,
		&wp.rcNormalPosition.right, &wp.rcNormalPosition.bottom);

	if (nRead != 10)
		return false;

	wp.length = sizeof wp;
	*pwp = wp;

	return true;
}

static void PASCAL NEAR WriteWindowPlacement(LPWINDOWPLACEMENT pwp)
{
	TCHAR szBuffer[sizeof("-32767")*8 + sizeof("65535")*2];

	wsprintf(szBuffer, szFormat,
		pwp->flags, pwp->showCmd,
		pwp->ptMinPosition.x, pwp->ptMinPosition.y,
		pwp->ptMaxPosition.x, pwp->ptMaxPosition.y,
		pwp->rcNormalPosition.left, pwp->rcNormalPosition.top,
		pwp->rcNormalPosition.right, pwp->rcNormalPosition.bottom);
	AfxGetApp()->WriteProfileString(szSection, szWindowPos, szBuffer);
}

static BOOL PASCAL NEAR ReadPreferences( CMainFrame* pFrame)
{	CDC pDisplay;
	if( pDisplay.CreateCompatibleDC(NULL) )
	{	int ScreenX = pDisplay.GetDeviceCaps(HORZRES);
		if( ScreenX>=1000 )
			pFrame->m_CommandSize = 1;
		else
			pFrame->m_CommandSize = 0;
	} else
		pFrame->m_CommandSize = 0;

	CString strBuffer = AfxGetApp()->GetProfileString(szSection, szPreferences);
	if( strBuffer.IsEmpty() )
	{	SeqFormat = false;
		pFrame->m_CopyZoom = 2;
		SetMouseMode(MMRasMol);
		pFrame->m_CommandId = 0;
		pFrame->m_HelpLine = "C:\\";
		return false;
	}

	int a,b,c,d,e;
	char f[512];
	int nRead = _stscanf(strBuffer, szPrefReadFormat, &a, &b, &c, &d, &e, &f);

	if( nRead!=6 )
	{	SeqFormat = false;
		pFrame->m_CopyZoom = 2;
		SetMouseMode(MMRasMol);
		pFrame->m_CommandId = 0;
		pFrame->m_HelpLine = "C:\\";
		return false;
	}
	
	pFrame->m_CopyZoom = a;
	SetMouseMode(b);	
	SeqFormat = c;
	pFrame->m_CommandSize = d;
	pFrame->m_CommandId = e;
	pFrame->m_HelpLine = f;

	return true;
}

static void PASCAL NEAR WritePreferences()
{
	TCHAR szBuffer[sizeof("-32767")*5 + 512];

	if( newpref== 0 ) return;

	GETFRAME
	wsprintf(szBuffer, szPrefWriteFormat,
		(int)pFrame->m_CopyZoom,
		(int)MouseMode,
		(int)SeqFormat,
		(int)pFrame->m_CommandSize,
		(int)pFrame->m_CommandId,
		(LPTSTR)pFrame->m_HelpLine.LockBuffer() );

	pFrame->m_HelpLine.UnlockBuffer();

	AfxGetApp()->WriteProfileString(szSection, szPreferences, szBuffer);
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNAMIC(CMainFrame, CMDIFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CMDIFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_WM_SYSCOMMAND( )
	ON_UPDATE_COMMAND_UI(ID_COMMANDVIEW, OnUpdateViewCommandBar)
	ON_COMMAND(ID_COMMANDVIEW, OnViewCommandBar)
	ON_MESSAGE(WM_COLOR, OnChangeColor)
	ON_COMMAND(ID_SELECTION_NEW, OnSelectionNew)
	ON_COMMAND(ID_SELECTION_ADD, OnSelectionAdd)
	ON_COMMAND(ID_SELECTION_REMOVE, OnSelectionRemove)
	ON_COMMAND(ID_SELECTION_EXTRACT, OnSelectionExtract)
	ON_COMMAND(ID_SELECTION_EXCLUDE, OnSelectionExclude)
	ON_UPDATE_COMMAND_UI(ID_SELECTION_NEW, OnUpdateSelectionNew)
	ON_UPDATE_COMMAND_UI(ID_SELECTION_ADD, OnUpdateSelectionAdd)
	ON_UPDATE_COMMAND_UI(ID_SELECTION_REMOVE, OnUpdateSelectionRemove)
	ON_UPDATE_COMMAND_UI(ID_SELECTION_EXTRACT, OnUpdateSelectionExtract)
	ON_UPDATE_COMMAND_UI(ID_SELECTION_EXCLUDE, OnUpdateSelectionExclude)
	ON_COMMAND(ID_VIEW_SELECTBAR, OnViewSelectbar)
	ON_UPDATE_COMMAND_UI(ID_VIEW_SELECTBAR, OnUpdateViewSelectbar)
	ON_COMMAND(ID_VIEW_COLORBAR, OnViewColorbar)
	ON_UPDATE_COMMAND_UI(ID_VIEW_COLORBAR, OnUpdateViewColorbar)
	ON_WM_CLOSE()
	ON_COMMAND(ID_VIEW_STATUS_BAR, OnViewStatusBar)
	ON_UPDATE_COMMAND_UI(ID_VIEW_STATUS_BAR, OnUpdateViewStatusBar)
	ON_COMMAND(ID_FILE_PREFERENCES, OnFilePreferences)
	ON_WM_GETMINMAXINFO()
	ON_COMMAND(ID_VIEW_MENU, OnViewMenus)
	ON_COMMAND(ID_VIEW_TOOLBAR, OnViewToolbar)
	ON_UPDATE_COMMAND_UI(ID_VIEW_TOOLBAR, OnUpdateViewToolbar)
	ON_UPDATE_COMMAND_UI(ID_VIEW_MENU, OnUpdateViewMenu)
	ON_UPDATE_COMMAND_UI(ID_SELECTION_EXCLUDE, OnUpdateSelectionExclude)
	ON_WM_HELPINFO()
	ON_COMMAND(ID_HELP_RASMOL, OnHelpRasTop)
	//}}AFX_MSG_MAP

	ON_MESSAGE(WM_UPDATERASVIEW, UpdateAllViews)
	ON_COMMAND_RANGE(ID_COLOR, ID_COLOR+MAX_COLORS, OnCommandColor)


END_MESSAGE_MAP()

static UINT indicators[] =
{	ID_SEPARATOR,
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

CMainFrame::CMainFrame()
{	//Window setting & ShowWindow
	m_WndRestoreInit = 0;

	//Default preferences	
	m_CopyZoom = 2;
	SetMouseMode(MMRasMol);	
    SeqFormat = true;
	m_CommandSize = 0;
	m_HelpLine = "C:\\";

	m_SelectString = "";
	PickMode = PickIdent;	

	m_szCommandText = "";
	InfoSilentFlag = false;
}

CMainFrame::~CMainFrame()	{}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{	cs.cx = 750;
	cs.cy = 550;
	return CMDIFrameWnd::PreCreateWindow(cs);
}

int CMainFrame::CreateCommandBar(UINT nID)
{	if( !m_wndCommandBar.Create(this, nID,
		CBRS_BOTTOM|CBRS_TOOLTIPS|CBRS_FLYBY, nID) )
	{	return 0;
	}

	/*Create Font*/
	int bigletter = 0;
	if( m_CommandSize==1 )	bigletter = 2;	

	CDC pDisplay;
	if( !pDisplay.CreateCompatibleDC(NULL) )
		return 0;

	LOGFONT logFont;
	memset(&logFont, 0, sizeof(logFont));
	logFont.lfWeight = FW_NORMAL;
	logFont.lfPitchAndFamily = DEFAULT_PITCH | FF_SWISS;
	lstrcpy( logFont.lfFaceName, _T("Arial") );
	int height = -MulDiv(7+bigletter, pDisplay.GetDeviceCaps(LOGPIXELSY), 72);
	logFont.lfHeight = height;
	if( !m_wndCommandBar.m_FontPannel.CreateFontIndirect(&logFont) )
		return 0;

	height = -MulDiv(5+bigletter, pDisplay.GetDeviceCaps(LOGPIXELSY), 72);
	logFont.lfHeight = height;
	if( !m_wndCommandBar.m_FontBuddy.CreateFontIndirect(&logFont) )
		return 0;

	if(!m_wndCommandBar.OnInitDialogBar() )
	{	TRACE0("Failed to init CommandBar\n");
		return 0; 
	}

	return 1;
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{	UINT nID_COMMANDBAR;
 
	if( CMDIFrameWnd::OnCreate(lpCreateStruct) == -1 )
		return -1;

	ReadPreferences(this);
	if( m_CommandSize==0 )
		nID_COMMANDBAR = IDD_COMMANDVIEW;
	else
		nID_COMMANDBAR = IDD_COMMANDVIEW_BIG;

	//Initial window placement
	WINDOWPLACEMENT wp;
	if( ReadWindowPlacement(&wp) )
	{	if( wp.showCmd == SW_SHOWMAXIMIZED )
			m_WndRestoreInit = 2;
		else
			m_WndRestoreInit = 1;
		SetWindowPlacement(&wp);
	}

	EnableDocking(CBRS_ALIGN_TOP);

	//Toolbar
	if (!m_wndToolBar.Create(this) || !m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{	TRACE0("Failed to create toolbar\n");
		return -1; 
	}
	m_wndToolBar.SetBarStyle(m_wndToolBar.GetBarStyle() | CBRS_ALIGN_TOP |
		CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_HIDE_INPLACE );
	m_wndToolBar.SetWindowText(_T("Main"));
	m_wndToolBar.EnableDocking(CBRS_ALIGN_TOP);

	//SelectBar
	if( !m_wndSelectBar.Init(this) )
	{	TRACE0("Failed to create editbar\n");
		return -1; 
	}
	m_wndSelectBar.SetWindowText(_T("Selection"));
	m_wndSelectBar.EnableDocking(CBRS_ALIGN_TOP);

	DockControlBar(&m_wndToolBar,AFX_IDW_DOCKBAR_TOP);
	DockSelectBar();

	//Status bar
	if( !m_wndStatusBar.Create(this, WS_CHILD | CBRS_BOTTOM) ||
		!m_wndStatusBar.SetIndicators(indicators, sizeof(indicators)/sizeof(UINT)) )
	{	TRACE0("Failed to create status bar\n");
		return -1;
	}
	BOOL bView = (AfxGetApp()->GetProfileInt(_T("General"),_T("Status"),1)!=0);
	ShowControlBar(&m_wndStatusBar, bView, false);
	bView = (AfxGetApp()->GetProfileInt(_T("General"),_T("Selection"),1)!=0);

	//Color bar
	if( !m_wndColorBar.Init(this) )
	{	TRACE0("Failed to create color toolbar\n");
		return -1; 
	}

	CPoint point(550,150);
	FloatControlBar(&m_wndColorBar,point,CBRS_ALIGN_TOP );
	ShowControlBar(&m_wndColorBar,false, true);

	//Command bar
	if( !CreateCommandBar(nID_COMMANDBAR) )
	{	TRACE0("Failed to create CommandBar\n");
		return -1; 
	}

	if( m_CommandId==(int)nID_COMMANDBAR )
	{	LoadBarState(_T("General"));
		if( bView )
			ShowControlBar(&m_wndSelectBar, bView, false);
		if( m_wndSelectBar.IsFloating() )
			m_wndSelectBar.SetBarPalette(true);
	}
	m_CommandId = nID_COMMANDBAR;

	return 0;
}

void CMainFrame::OnSysCommand( UINT nID, LPARAM lParam )
{	
	CMDIFrameWnd::OnSysCommand( nID, lParam );

	static BOOL IconeFlag = false;

	//Allow decent restoring if initially maximized
	int x = ::GetSystemMetrics(SM_CXMAXIMIZED);
	int y = ::GetSystemMetrics(SM_CYMAXIMIZED);
	x = 750>x? x : 750;
	y = 500>y? y : 500;
	if( ((nID & 0xFFF0) == SC_RESTORE) && (m_WndRestoreInit == 2 ) && !IconeFlag )
	{	m_WndRestoreInit = 1;	//deactivate
		MoveWindow( 25, 25, x, y, TRUE );
	}

	IconeFlag = ((nID & 0xFFF0) == SC_MINIMIZE);

	//Stop spinmode on screensaver
	if( (nID & 0xFFF0) == SC_SCREENSAVE || (nID & 0xFFF0) == SC_MINIMIZE ) 
			m_wndCommandBar.ResetSpinMode();
}

void CMainFrame::OnGetMinMaxInfo(MINMAXINFO FAR* lpMMI) 
{
    POINT p;
	p.x = ::GetSystemMetrics(SM_CXMAXIMIZED);
	p.y = ::GetSystemMetrics(SM_CYMAXIMIZED);
	lpMMI->ptMaxSize = p;
	p.x = - ::GetSystemMetrics(SM_CXSIZEFRAME);
	p.y = - ::GetSystemMetrics(SM_CYSIZEFRAME);
    lpMMI->ptMaxPosition = p;
	
	CMDIFrameWnd::OnGetMinMaxInfo(lpMMI);
}

void CMainFrame::OnClose() 
{	SaveBarState(_T("General"));
	AfxGetApp()->WriteProfileInt( _T("General"),_T("Status"),
		(m_wndStatusBar.GetStyle() & WS_VISIBLE) != 0);
	AfxGetApp()->WriteProfileInt( _T("General"),_T("Selection"),
		(m_wndSelectBar.GetStyle() & WS_VISIBLE) != 0);

	WINDOWPLACEMENT wp;
	wp.length = sizeof wp;
	if( GetWindowPlacement(&wp) )
	{	wp.flags = 0;
		if (IsZoomed())
			wp.flags |= WPF_RESTORETOMAXIMIZED;
		WriteWindowPlacement(&wp);
	}

	//Cannot save empty preference
	if( m_HelpLine.IsEmpty() )
		m_HelpLine = "C:\\";

	WritePreferences();

	CMDIFrameWnd::OnClose();
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{	CMDIFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{	CMDIFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
//Bar commands

/*ToolBar*/
void CMainFrame::OnViewToolbar() 
{	BOOL bView = (m_wndToolBar.GetStyle() & WS_VISIBLE);
	ShowControlBar(&m_wndToolBar, !bView, false);
	if( !bView )
		DisableMenu = false;
}

void CMainFrame::OnUpdateViewToolbar(CCmdUI* pCmdUI) 
{	pCmdUI->SetCheck( (m_wndToolBar.GetStyle() & WS_VISIBLE) != 0 );	}

/*SelectBar*/
void CMainFrame::OnViewSelectbar() 
{	BOOL bView = (m_wndSelectBar.GetStyle() & WS_VISIBLE);
	ShowControlBar(&m_wndSelectBar, !bView, false);

	if( !bView )
		DisableMenu = false;
}

void CMainFrame::OnUpdateViewSelectbar(CCmdUI* pCmdUI) 
{	pCmdUI->SetCheck( (m_wndSelectBar.GetStyle() & WS_VISIBLE) != 0 );	}

void CMainFrame::DockSelectBar()
{	CRect rect;
	RecalcLayout();
	m_wndToolBar.GetWindowRect(&rect);
	rect.OffsetRect(1,0);
	DockControlBar(&m_wndSelectBar,AFX_IDW_DOCKBAR_TOP,&rect);
}

/*ColorBar*/
void CMainFrame::OnViewColorbar() 
{	BOOL bView = (m_wndColorBar.GetStyle() & WS_VISIBLE);
	ShowControlBar(&m_wndColorBar, !bView, false);
}

void CMainFrame::OnUpdateViewColorbar(CCmdUI* pCmdUI) 
{	pCmdUI->SetCheck( (m_wndColorBar.GetStyle() & WS_VISIBLE) != 0 );	}

/*CommandBar*/
void CMainFrame::OnViewCommandBar() 
{	BOOL bView = (m_wndCommandBar.GetStyle() & WS_VISIBLE);
	ShowControlBar(&m_wndCommandBar, !bView, false);
	if( !bView )
		DisableMenu = false;
}

void CMainFrame::ActivateCommandBar()
{	ShowControlBar(&m_wndCommandBar, true, false);	
	DisableMenu = false;
}

void CMainFrame::DesactivateCommandBar()
{	ShowControlBar(&m_wndCommandBar, false, false);	}

void CMainFrame::OnUpdateViewCommandBar(CCmdUI* pCmdUI) 
{	pCmdUI->SetCheck( (m_wndCommandBar.GetStyle() & WS_VISIBLE) != 0 );	}

/*StatusBar*/
void CMainFrame::OnViewStatusBar() 
{	BOOL bView = (m_wndStatusBar.GetStyle() & WS_VISIBLE);
	ShowControlBar(&m_wndStatusBar, !bView, false);
	if( !bView )
		DisableMenu = false;
}

void CMainFrame::OnUpdateViewStatusBar(CCmdUI* pCmdUI) 
{	pCmdUI->SetCheck( (m_wndStatusBar.GetStyle() & WS_VISIBLE) != 0 );	}

/*Full screen*/

//Here is the code for a true full screen
//void CRasView::OnFullScreen() 
//{	int xBorder = GetSystemMetrics(SM_CXSIZEFRAME) + 1;
//    int yBorder = GetSystemMetrics(SM_CXSIZEFRAME) + 1;
//    int hCaption = GetSystemMetrics(SM_CYCAPTION) + 1;
//    int hMenu = GetSystemMetrics(SM_CYMENU) + 1;
//    int width = GetSystemMetrics(SM_CXSCREEN) + 2*xBorder;
//    int height = GetSystemMetrics(SM_CYSCREEN) + 2*yBorder+hCaption+hMenu;
//    int yPos = -(hCaption + hMenu + yBorder);
//    int xPos = -xBorder;
//    GetParent()->MoveWindow(xPos, yPos, width, height, TRUE);
//}
//
//void CMainFrame::OnGetMinMaxInfo(MINMAXINFO FAR* lpMMI)
//{	lpMMI->ptMaxSize.x = 0x7000;
//    lpMMI->ptMaxSize.y = 0x7000;
//    lpMMI->ptMaxTrackSize.x = 0x7000;
//    lpMMI->ptMaxTrackSize.y = 0x7000;
//}

void CMainFrame::OnViewMenus() 
{	static BOOL ToolBarFlag = true;
	static BOOL SelectBarFlag = true;
	static BOOL CommandBarFlag = true;
	static BOOL StatusBarFlag = true;

	//DisableMenu always False at beginning
	DisableMenu = !DisableMenu;

	if( DisableMenu )
	{	//memorize status
		ToolBarFlag = m_wndToolBar.GetStyle() & WS_VISIBLE;
		SelectBarFlag = m_wndSelectBar.GetStyle() & WS_VISIBLE;
		CommandBarFlag = m_wndCommandBar.GetStyle() & WS_VISIBLE;
		StatusBarFlag = m_wndStatusBar.GetStyle() & WS_VISIBLE;

		if( ToolBarFlag )		ShowControlBar(&m_wndToolBar, false, false);
		if( SelectBarFlag )		
			if( !m_wndSelectBar.IsFloating() )
								ShowControlBar(&m_wndSelectBar, false, false);
		if( CommandBarFlag )	ShowControlBar(&m_wndCommandBar, false, false);
		if( StatusBarFlag )		ShowControlBar(&m_wndStatusBar, false, false);
	} else
	{	if( ToolBarFlag )		ShowControlBar(&m_wndToolBar, true, false);

		if( SelectBarFlag )		ShowControlBar(&m_wndSelectBar, true, false);
		if( CommandBarFlag && GetActiveRasView() )
								ShowControlBar(&m_wndCommandBar, true, false);
		if( StatusBarFlag )		ShowControlBar(&m_wndStatusBar, true, false);
	}

	RecalcLayout();
}

void CMainFrame::EnableMenus( int flag )
{	if( DisableMenu==flag )
		OnViewMenus();
}

void CMainFrame::OnUpdateViewMenu(CCmdUI* pCmdUI) 
{	pCmdUI->SetCheck( DisableMenu );	}

/////////////////////////////////////////////////////////////////////////////

/*Active RasView*/
CRasView* CMainFrame::GetActiveRasView()
{	CMDIChildWnd *pChild = (CMDIChildWnd *) GetActiveFrame();
	if ( pChild == NULL )
		return NULL;
	CRasView *pView = (CRasView *) pChild->GetActiveView();
	if( pView == NULL )
		return NULL;
	ASSERT(pView->IsKindOf(RUNTIME_CLASS(CRasView)));
	return pView;
}

void CMainFrame::OnCommandColor(UINT nID)
{	m_wndColorBar.OnCommandHandler(nID);
}

LRESULT CMainFrame::OnChangeColor(WPARAM wParam, LPARAM lParam )
{	m_crColour = (COLORREF) wParam;
	return 0L;
}

/*Selections*/
void CMainFrame::SetSelection( int nMode, CString& sExpression)
{	if( Database == NULL )
		return;
	CRasView* pView = GetActiveRasView();
	if( pView == NULL )
		return;
	if( sExpression.IsEmpty() )
		return;

	switch(nMode)
	{ case(SELECTION_EMPTY)	:
	  case(SELECTION_NEW)	:	
		pView->ExecuteCommand( DisplayOn, "select (" + sExpression + ")\n" );
		break;
	  case(SELECTION_ADD)	:	
		pView->ExecuteCommand( DisplayOn, "select (" + sExpression + \
		") || selected\n" );
		break;
	  case(SELECTION_REMOVE):	
		pView->ExecuteCommand( DisplayOn, "select selected and not ("	+ \
		sExpression + ")\n" );
		break;
	  case(SELECTION_EXTRACT):	
		pView->ExecuteCommand( DisplayOn,"select (" + sExpression + \
		") && selected\n" );
		break;
	  case(SELECTION_EXCLUDE):		
		pView->ExecuteCommand( DisplayOn, 
							"select (" + sExpression + ") and not selected\n" );
		break;
	  default				:
		return;
	}
}

void CMainFrame::OnSelectionNew()
{	if( m_SelectString.IsEmpty() )
		return;
	SetSelection( SELECTION_NEW, m_SelectString );
}

void CMainFrame::OnUpdateSelectionNew(CCmdUI* pCmdUI)
{	pCmdUI->Enable( Database != NULL );	}

void CMainFrame::OnSelectionAdd()
{	if( m_SelectString.IsEmpty() )
		return;
	SetSelection( SELECTION_ADD, m_SelectString );
}

void CMainFrame::OnUpdateSelectionAdd(CCmdUI* pCmdUI)
{	pCmdUI->Enable( Database != NULL );	}

void CMainFrame::OnSelectionRemove()
{	if( m_SelectString.IsEmpty() )
		return;
	SetSelection( SELECTION_REMOVE, m_SelectString );
}

void CMainFrame::OnUpdateSelectionRemove(CCmdUI* pCmdUI)
{	pCmdUI->Enable( Database != NULL );	}

void CMainFrame::OnSelectionExtract()
{	if( m_SelectString.IsEmpty() )
		return;
	SetSelection( SELECTION_EXTRACT, m_SelectString );
}

void CMainFrame::OnUpdateSelectionExtract(CCmdUI* pCmdUI)
{	pCmdUI->Enable( Database != NULL );	}

void CMainFrame::OnSelectionExclude()
{	if( m_SelectString.IsEmpty() )
		return;
	SetSelection(SELECTION_EXCLUDE, m_SelectString);
}

void CMainFrame::OnUpdateSelectionExclude(CCmdUI* pCmdUI)
{	pCmdUI->Enable( Database != NULL );	}

/*Preferences*/
void CMainFrame::OnFilePreferences() 
{	CDialogPreference dlg;
	dlg.m_MouseRasmol  = (MouseMode == MMRasMol);
	dlg.m_MouseQuanta  = (MouseMode == MMQuanta);
	dlg.m_MouseInsight = (MouseMode == MMInsight);
	dlg.m_MouseSybyl   = (MouseMode == MMSybyl);
	dlg.m_ZoomEdit = m_CopyZoom;
	dlg.m_SequenceTrue = SeqFormat;
	dlg.m_SequenceFalse = !SeqFormat;
	dlg.m_PannelSize = m_CommandSize;
	dlg.m_HelpSource = m_HelpLine;

	if( dlg.DoModal()!=IDOK )
		return;
	else
		newpref = 1;

	if( dlg.m_MouseRasmol )		SetMouseMode(MMRasMol);
	if( dlg.m_MouseQuanta )		SetMouseMode(MMQuanta);
	if( dlg.m_MouseInsight )	SetMouseMode(MMInsight);
	if( dlg.m_MouseSybyl )		SetMouseMode(MMSybyl);

	m_CopyZoom = dlg.m_ZoomEdit;
	SeqFormat = dlg.m_SequenceTrue;

	//Change commandbar size
	if( m_CommandSize!=dlg.m_PannelSize )
	{	m_CommandSize = dlg.m_PannelSize;
		m_wndCommandBar.~CPannelBar();
		m_wndCommandBar.CPannelBar::CPannelBar();
		if( m_CommandSize==0 )
			CreateCommandBar(IDD_COMMANDVIEW);
		else
			CreateCommandBar(IDD_COMMANDVIEW_BIG);
		ShowControlBar(&m_wndCommandBar, true, false);
	}
	
	m_HelpLine = dlg.m_HelpSource;	
}

BOOL CMainFrame::OnCommand(WPARAM wParam, LPARAM lParam) 
{	int nDoc = 0;
	CMDIChildWnd *pChild = (CMDIChildWnd *) GetActiveFrame();
	CRasDoc* pDoc = (CRasDoc *) pChild->GetActiveDocument( );
	if( pDoc != NULL )
	{	CDocTemplate* pDocTemp;
		pDocTemp = pDoc->GetDocTemplate( );
		POSITION posdoc = pDocTemp->GetFirstDocPosition( );
		while(posdoc != NULL)
		{	pDoc = (CRasDoc *)pDocTemp->GetNextDoc(posdoc);
			nDoc++;
		}
	}

	//0x0000ff00 = 65280 - allow document switch through the menu when pause
	BOOL b = 
		(int) LOWORD(wParam) >= 65280 && (int) LOWORD(wParam) <= (65280 + nDoc -1);
	//allow to switch command pannel if paused
	if( (int) LOWORD(wParam)==ID_COMMANDVIEW )
		b = true;
	//same for full screen
	if( (int) LOWORD(wParam)==ID_VIEW_MENU )
		b = true;

	BOOL c = IsPaused && !b && !( (int) HIWORD(wParam) );
	if( c )
	{	WriteString("Type any key to continue the script or <Esc> to terminate\n");
		return true;
	}
	
	return CMDIFrameWnd::OnCommand(wParam, lParam);
}

LRESULT CMainFrame::UpdateAllViews(WPARAM wParam, LPARAM lParam)
{
	CRasView* pPrevView = NULL;
	CRasView* pNextView = NULL;
	CRasDoc* pPrevDoc;
	CRasDoc* pNextDoc;

	CMDIChildWnd *pChild = (CMDIChildWnd *) GetActiveFrame();
	pPrevDoc = (CRasDoc *) pChild->GetActiveDocument( );
	CDocTemplate* pDocTemp;
	pDocTemp = pPrevDoc->GetDocTemplate( );
	POSITION posdoc = pDocTemp->GetFirstDocPosition( );

	//Activate all views successively
	//to cause redrawing of each database.
	while( posdoc!=NULL )
	{	//Save data from previous
		pPrevView = GetActiveRasView();
		ASSERT( pPrevView!=NULL );
		if( pPrevView!=NULL )
		{	pPrevDoc = (CRasDoc *)pPrevView->GetDocument();
			pPrevDoc->ManageDisplay(false);
		}

		//Get the doc and the unique view
		pNextDoc = (CRasDoc *)pDocTemp->GetNextDoc(posdoc);
		POSITION posview = pNextDoc->GetFirstViewPosition( );
		pNextView = (CRasView *)pNextDoc->GetNextView( posview );
		ASSERT( posview==0 && pNextView!=NULL );

		pChild = (CMDIChildWnd *)pNextView->GetParentFrame( );
		pChild->MDIActivate();

		//Pseudo Onsize
		CRect rec;
		pNextView->GetWindowRect( (LPRECT) rec );
		if( int dx = rec.Width( )%4 )
			rec.right += 4-dx;
		pNextDoc->m_DataDisplay.Data_XRange = rec.Width( );
		pNextDoc->m_DataDisplay.Data_YRange = rec.Height( );
		pNextDoc->FixRange();
		if( Database )
			ReDrawFlag |= RFReSize;
		pNextView->RefreshView();	
	}

	return 0L;
}

//Get available client area in screen coordinates
CRect CMainFrame::GetScreenClientCoor()
{	CRect rRecFrame;
	CRect rect;

	GetClientRect( (LPRECT) rRecFrame );
	ClientToScreen( (LPRECT) rRecFrame );

	if( (m_wndToolBar.GetStyle() & WS_VISIBLE) != 0 &&
		 !m_wndToolBar.IsFloating( ) )
	{	m_wndToolBar.GetWindowRect(&rect);
		rRecFrame.top = rect.bottom ;
	}

	if( (m_wndSelectBar.GetStyle() & WS_VISIBLE) != 0 &&
		 !m_wndSelectBar.IsFloating( ) )
	{	m_wndSelectBar.GetWindowRect(&rect);
		if( rect.bottom > rRecFrame.top )
			rRecFrame.top = rect.bottom ;
	}

	if( (m_wndCommandBar.GetStyle() & WS_VISIBLE) != 0 )
	{	m_wndCommandBar.GetWindowRect( &rect );
		rRecFrame.bottom = rect.top;
	} else if( (m_wndStatusBar.GetStyle() & WS_VISIBLE) != 0 )
	{	m_wndStatusBar.GetWindowRect( &rect );
		rRecFrame.bottom = rect.top;
	}

	return(rRecFrame);
}

/*======*/
/* Help */
/*======*/

void CMainFrame::OnHelpRasTop() 
{	InitHelpHTML();	}

/* Handle F1 key */
BOOL CMainFrame::OnHelpInfo(HELPINFO* pHelpInfo) 
{	OnHelpRasTop();
	return true;
}

