/***************************************************************************
 *                                                                         *
 *                               RasTop 1.3.1                              *
 *                 Molecular Graphics Visualisation Tool                   *
 *                          by Philippe Valadon                            *
 *                              21 Mars 2001                               *
 *                                                                         *
 *                                *******                                  *
 *                                                                         *
 *                   Based on RasMol 2.6 by Roger Sayle                    *
 * Biomolecular Structures Group, Glaxo Wellcome Research & Development,   *
 *                      Stevenage, Hertfordshire, UK                       *
 *         Version 2.6, August 1995, Version 2.6.4, December 1998          *
 *                   Copyright (C) Roger Sayle 1992-1999                   *
 *                                                                         *
 *                  and Based on Mods by Arne Mueller                      *
 *                      Version 2.6x1, May 1998                            *
 *                   Copyright (C) Arne Mueller 1998                       *
 *                                                                         *
 *    and Based on Version 2.7.0, 2.7.1 Mods by Herbert J. Bernstein       *
 *           Bernstein + Sons, P.O. Box 177, Bellport, NY, USA             *
 *                      yaya@bernstein-plus-sons.com                       *
 *                    2.7.0 March 1999, 2.7.1 June 1999                    *
 *              Copyright (C) Herbert J. Bernstein 1998-1999               *
 *                                                                         *
 *                RasTop Version 1.3.x by Philippe Valadon                 *
 *                 P.O. Box 13014, La Jolla, CA 92039, USA,                *
 *                         valadon@ix.netcom.com                           *
 *                Copyright (C) Philippe Valadon 2000-2001                 *
 *                                                                         *
 *                                *******                                  *
 *                                                                         *
 * As written by Herbert Bernstein in Rasmol Version 2.7.1:                *
 *                                                                         *
 * Please read the file NOTICE for important notices which apply to this   *
 * package. If you are not going to make changes to Rasmol, you are not    *
 * only permitted to freely make copies and distribute them, you are       *
 * encouraged to do so, provided you do the following:                     *
 *   * 1. Either include the complete documentation, especially the file   *
 *     NOTICE, with what you distribute or provide a clear indication      *
 *     where people can get a copy of the documentation; and               *
 *   * 2. Please give credit where credit is due citing the version and    *
 *     original authors properly; and                                      *
 *   * 3. Please do not give anyone the impression that the original       *
 *     authors are providing a warranty of any kind.                       *
 *                                                                         *
 * If you would like to use major pieces of Rasmol in some other program,  *
 * make modifications to Rasmol, or in some other way make what a lawyer   *
 * would call a "derived work", you are not only permitted to do so, you   *
 * are encouraged to do so. In addition to the things we discussed above,  *
 * please do the following:                                                *
 *   * 4. Please explain in your documentation how what you did differs    *
 *     from this version of Rasmol; and                                    *
 *   * 5. Please make your modified source code available.                 *
 *                                                                         *
 * This version of Rasmol is not in the public domain, but it is given     *
 * freely to the community in the hopes of advancing science. If you make  *
 * changes, please make them in a responsible manner, and please offer us  *
 * the opportunity to include those changes in future versions of RasTop.  *
 ***************************************************************************/

/////////////////////////////////////////////////////////////////////////////
// mainfrm.h 
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_MAINFRM_H_INCLUDED_)
#define AFX_MAINFRM_H_INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#define		SELECTION_EMPTY		0
#define		SELECTION_NEW		1
#define		SELECTION_ADD		2
#define		SELECTION_REMOVE	3
#define		SELECTION_EXTRACT	4
#define		SELECTION_EXCLUDE	5

#include "pannel.h"
#include "selectbar.h"
#include "colorbar.h"


/////////////////////////////////////////////////////////////////////////////
// CMainFrame mainframe

class CMainFrame : public CMDIFrameWnd
{
	DECLARE_DYNAMIC(CMainFrame)
public:
	CMainFrame();

// Attributes
public:

// Operations
public:
	// Overrides
	//{{AFX_VIRTUAL(CMainFrame)
	public:
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	protected:
	virtual BOOL OnCommand(WPARAM wParam, LPARAM lParam);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CMainFrame();
	void DockSelectBar();
	void SetSelection( int nMode, CString& sExpression);

#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected: 
	CToolBar		  m_wndToolBar;
	CStatusBar		  m_wndStatusBar;

public:	
	int			m_WndRestoreInit;
	int			m_CommandSize;
	int			m_CommandId;
	int			m_CopyZoom;
	CString		m_HelpLine;
	int			m_MenuPicker;
	CPannelBar  m_wndCommandBar;
	CSelectBar	m_wndSelectBar;
	CColorBar	m_wndColorBar;
	CString		m_SelectString;
	COLORREF	m_crColour;
	CString		m_szCommandText;
	int			InfoSilentFlag;

	void UpdateAllViews( CRasView* pIni);
	void ActivateCommandBar();
	void DesactivateCommandBar();
	CRasView* GetActiveRasView();
	CRect GetScreenClientCoor();
	void EnableMenus( int flag );

protected:

	int CreateCommandBar( UINT nID );

	//{{AFX_MSG(CMainFrame)
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnSysCommand( UINT nID, LPARAM lParam );	
	afx_msg void OnUpdateViewCommandBar(CCmdUI* pCmdUI);
	afx_msg void OnViewCommandBar();
	afx_msg LRESULT OnChangeColor(WPARAM wParam, LPARAM lParam);
	afx_msg void OnSelectionNew();
	afx_msg void OnSelectionAdd();
	afx_msg void OnSelectionRemove();
	afx_msg void OnSelectionExtract();
	afx_msg void OnSelectionExclude();
	afx_msg void OnUpdateSelectionNew(CCmdUI* pCmdUI);
	afx_msg void OnUpdateSelectionAdd(CCmdUI* pCmdUI);
	afx_msg void OnUpdateSelectionRemove(CCmdUI* pCmdUI);
	afx_msg void OnUpdateSelectionExtract(CCmdUI* pCmdUI);
	afx_msg void OnUpdateSelectionExclude(CCmdUI* pCmdUI);
	afx_msg void OnViewSelectbar();
	afx_msg void OnUpdateViewSelectbar(CCmdUI* pCmdUI);
	afx_msg void OnViewColorbar();
	afx_msg void OnUpdateViewColorbar(CCmdUI* pCmdUI);
	afx_msg void OnClose();
	afx_msg void OnViewStatusBar();
	afx_msg void OnUpdateViewStatusBar(CCmdUI* pCmdUI);
	afx_msg void OnFilePreferences();
	afx_msg void OnGetMinMaxInfo(MINMAXINFO FAR* lpMMI);
	afx_msg void OnViewMenus();
	afx_msg void OnViewToolbar();
	afx_msg void OnUpdateViewToolbar(CCmdUI* pCmdUI);
	afx_msg void OnUpdateViewMenu(CCmdUI* pCmdUI);
	afx_msg void OnHelpRasTop();
	afx_msg BOOL OnHelpInfo(HELPINFO* pHelpInfo);
	//}}AFX_MSG

protected:
	afx_msg LRESULT UpdateAllViews(WPARAM wParam, LPARAM lParam);
	afx_msg void OnCommandColor(UINT nID);
	DECLARE_MESSAGE_MAP()
};

//{{AFX_INSERT_LOCATION}}

#endif // !defined(AFX_MAINFRM_H_INCLUDED_)
