/***************************************************************************
 *                                                                         *
 *                               RasTop 1.3.1                              *
 *                 Molecular Graphics Visualisation Tool                   *
 *                          by Philippe Valadon                            *
 *                              21 Mars 2001                               *
 *                                                                         *
 *                                *******                                  *
 *                                                                         *
 *                   Based on RasMol 2.6 by Roger Sayle                    *
 * Biomolecular Structures Group, Glaxo Wellcome Research & Development,   *
 *                      Stevenage, Hertfordshire, UK                       *
 *         Version 2.6, August 1995, Version 2.6.4, December 1998          *
 *                   Copyright (C) Roger Sayle 1992-1999                   *
 *                                                                         *
 *                  and Based on Mods by Arne Mueller                      *
 *                      Version 2.6x1, May 1998                            *
 *                   Copyright (C) Arne Mueller 1998                       *
 *                                                                         *
 *    and Based on Version 2.7.0, 2.7.1 Mods by Herbert J. Bernstein       *
 *           Bernstein + Sons, P.O. Box 177, Bellport, NY, USA             *
 *                      yaya@bernstein-plus-sons.com                       *
 *                    2.7.0 March 1999, 2.7.1 June 1999                    *
 *              Copyright (C) Herbert J. Bernstein 1998-1999               *
 *                                                                         *
 *                RasTop Version 1.3.x by Philippe Valadon                 *
 *                 P.O. Box 13014, La Jolla, CA 92039, USA,                *
 *                         valadon@ix.netcom.com                           *
 *                Copyright (C) Philippe Valadon 2000-2001                 *
 *                                                                         *
 *                                *******                                  *
 *                                                                         *
 * As written by Herbert Bernstein in Rasmol Version 2.7.1:                *
 *                                                                         *
 * Please read the file NOTICE for important notices which apply to this   *
 * package. If you are not going to make changes to Rasmol, you are not    *
 * only permitted to freely make copies and distribute them, you are       *
 * encouraged to do so, provided you do the following:                     *
 *   * 1. Either include the complete documentation, especially the file   *
 *     NOTICE, with what you distribute or provide a clear indication      *
 *     where people can get a copy of the documentation; and               *
 *   * 2. Please give credit where credit is due citing the version and    *
 *     original authors properly; and                                      *
 *   * 3. Please do not give anyone the impression that the original       *
 *     authors are providing a warranty of any kind.                       *
 *                                                                         *
 * If you would like to use major pieces of Rasmol in some other program,  *
 * make modifications to Rasmol, or in some other way make what a lawyer   *
 * would call a "derived work", you are not only permitted to do so, you   *
 * are encouraged to do so. In addition to the things we discussed above,  *
 * please do the following:                                                *
 *   * 4. Please explain in your documentation how what you did differs    *
 *     from this version of Rasmol; and                                    *
 *   * 5. Please make your modified source code available.                 *
 *                                                                         *
 * This version of Rasmol is not in the public domain, but it is given     *
 * freely to the community in the hopes of advancing science. If you make  *
 * changes, please make them in a responsible manner, and please offer us  *
 * the opportunity to include those changes in future versions of RasTop.  *
 ***************************************************************************/

////////////////////////////////////////////////////////////////////////////
// pannel.cpp:
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "pannel.h"
#include "RasTop.h"
#include "rasdoc.h"
#include "rasview.h"
#include "mainfrm.h"
#include "graphics.h"


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define LINECOUNTMAX 100

//////////////////////////////////////////////////////////////////////
// CPannelBar

IMPLEMENT_DYNAMIC(CPannelBar, CDialogBar)

BEGIN_MESSAGE_MAP(CPannelBar, CDialogBar)
	//{{AFX_MSG_MAP(CPannelBar)
	ON_WM_HSCROLL()
	ON_WM_TIMER()
	ON_BN_CLICKED(IDC_RADIO_ROTATE, OnRotateButton)
	ON_BN_CLICKED(IDC_RADIO_TRANSLATE, OnTranszoomButton)
	ON_COMMAND(IDC_BUTTON_AUTO, OnSpinButton)
	ON_BN_CLICKED(IDC_BUTTON_RESET, OnResetButton)
	ON_BN_CLICKED(IDC_BUTTON_CLOSE_COMMAND, OnCloseButton)
	ON_UPDATE_COMMAND_UI(IDC_BUTTON_CLOSE_COMMAND, OnUpdateCloseButton)
	ON_UPDATE_COMMAND_UI(IDC_SLID_SLAB, OnUpdateSlabSlid)
	ON_UPDATE_COMMAND_UI(IDC_SLID_DEPTH, OnUpdateDepthSlid)
	ON_UPDATE_COMMAND_UI(IDC_SLID_LIGHT, OnUpdateLightSlid)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

CPannelBar::CPannelBar()	{}
CPannelBar::~CPannelBar()	{}

BOOL CPannelBar::Create(CWnd * pParentWnd, UINT nIDTemplate, UINT nStyle, UINT nID)
{	if( !CDialogBar::Create(pParentWnd, MAKEINTRESOURCE(nIDTemplate), nStyle, nID) )
		return false;
	return TRUE;
}

BOOL CPannelBar::OnInitDialogBar()
{

	CWnd* pWndControl =  GetDlgItem((int)IDC_STATIC_SLAB);
	pWndControl->SetFont( &m_FontPannel, true );
	pWndControl =  GetDlgItem((int)IDC_STATIC_DEPTH);
	pWndControl->SetFont( &m_FontPannel, true );
	pWndControl =  GetDlgItem((int)IDC_STATIC_LIGHT);
	pWndControl->SetFont( &m_FontPannel, true );
	pWndControl =  GetDlgItem((int)IDC_STATIC_SPECULAR);
	pWndControl->SetFont( &m_FontPannel, true );
	pWndControl =  GetDlgItem((int)IDC_STATIC_SHADE);
	pWndControl->SetFont( &m_FontPannel, true );

	m_RasmolStatus.SubclassDlgItem(IDC_RASMOL_STATUS, this);
	StatusLight.LoadBitmap(IDB_LIGHT_ON);
	m_RasmolStatus.SetBitmap(StatusLight.operator HBITMAP());

	m_Infoline.SubclassDlgItem(IDC_INFOLINE,this);
	m_Infoline.SetReadOnly(true );
	m_Infoline.SetFont(&m_FontPannel, true);

	infopointer.SubclassDlgItem(IDC_INFOPOINTER,this);
	infopointeroff.LoadBitmap( IDB_INFOPOINTEROFF );
	infopointer.SetBitmap(infopointeroff.operator HBITMAP());
	m_PannelTimerID_1 = 0;
	m_InfoTimerStatus = true;

	m_ScrollX.SubclassDlgItem (IDC_ROTATEX,this);
	m_ScrollX.SetScrollRange(0,200,true);
	m_ScrollX.SetScrollPos(100,true);

	m_ScrollY.SubclassDlgItem(IDC_ROTATEY,this);
	m_ScrollY.SetScrollRange(0,200,true);
	m_ScrollY.SetScrollPos(100,true);

	m_ScrollZ.SubclassDlgItem(IDC_ROTATEZ,this);
	m_ScrollZ.SetScrollRange(0,200,true);
	m_ScrollZ.SetScrollPos(100,true);

	m_moderotate.SubclassDlgItem(IDC_RADIO_ROTATE,this);
	m_moderotate.SetCheck(true );
	m_modetranslate.SubclassDlgItem(IDC_RADIO_TRANSLATE,this);
	m_modetranslate.SetCheck(false );
	m_moderotate.SetFont(&m_FontPannel,true);
	m_modetranslate.SetFont(&m_FontPannel,true);
	
	Shadow_Flag = false;
	FontStroke_Flag = 0;

	m_reset.SubclassDlgItem(IDC_BUTTON_RESET,this);
	m_reset.SetState(false);
	m_reset.SetCheck(false);
	m_reset.SetFont(&m_FontPannel,true);

	m_modespin.SubclassDlgItem(IDC_BUTTON_AUTO,this);
	m_modespin.SetState(false);
	m_modespin.SetCheck(false);
	SpinX = SpinY = SpinZ = 0;
	m_PannelTimerID_0 = 0;
	m_spinStatus = false;
	m_modespin.SetFont(&m_FontPannel,true);

	m_Slab.SubclassDlgItem(IDC_SLID_SLAB, this);
	m_Slab.SetRange(0,200,false);
	m_Slab.SetTicFreq(20);
	m_Slab.SetLineSize(1);
	m_Slab.SetPageSize(10);
	m_Slab.SetPos(0);
	m_Slab.SetRange(0,200,true);

	m_Depth.SubclassDlgItem(IDC_SLID_DEPTH, this);
	m_Depth.SetRange(0,200,false);
	m_Depth.SetTicFreq(20);
	m_Depth.SetLineSize(1);
	m_Depth.SetPageSize(10);
	m_Depth.SetPos(200);
	m_Depth.SetRange(0,200,true);

	m_Light.SubclassDlgItem(IDC_SLID_LIGHT, this );
	m_Light.SetRange(0,100,false);
	m_Light.SetTicFreq(10);
	m_Light.SetLineSize(1);
	m_Light.SetPageSize(5);
	m_Light.SetPos((int)(DefaultAmbient*100));
	m_Light.SetRange(0,100,true);

	m_Specular.SubclassDlgItem(IDC_SPIN_SPECULAR,this);
	m_Specular.SetRange(14,0);
	m_Specular.SetPos(14);

	m_BuddySpec.SubclassDlgItem(IDC_BUDDY_SPEC,this);
	m_BuddySpec.SetReadOnly(true);
	m_BuddySpec.SetFont(&m_FontBuddy, true);
	m_BuddySpec.SetWindowText("off");

	m_Shade.SubclassDlgItem(IDC_SPIN_SHADE,this);
	m_Shade.SetRange(40,0);
	m_Shade.SetPos(20);

	m_BuddyShade.SubclassDlgItem(IDC_BUDDY_SHADE,this);
	m_BuddyShade.SetReadOnly(true);
	m_BuddyShade.SetFont(&m_FontBuddy, true);
	m_BuddyShade.SetWindowText("0");

	m_CloseCommandView.SubclassDlgItem(IDC_BUTTON_CLOSE_COMMAND,this);
	m_CloseCommandView.LoadBitmaps(IDB_COMMAND_CLOSEU,IDB_COMMAND_CLOSED,0,0);

	return true;
}

void CPannelBar::OnAddInfoline(CString& strText) 
{	
	AppendString (strText);	

	if( m_InfoTimerStatus )
	{	m_InfoTimerStatus = false;
		infopointeroff.DeleteObject();
		infopointeroff.LoadBitmap(IDB_INFOPOINTERON);
		infopointer.SetBitmap(infopointeroff.operator HBITMAP());
		m_PannelTimerID_1 = SetTimer(IDT_TIMER_1,350,NULL);
		if( m_PannelTimerID_1 == 0 ) 
		{	infopointeroff.DeleteObject();
			infopointeroff.LoadBitmap(IDB_INFOPOINTEROFF);
			infopointer.SetBitmap(infopointeroff.operator HBITMAP());
		}
	}
}

void CPannelBar::AppendString(CString str)
{	static InfoLineCount = 0;
	CString   strBuffer;
	int i; 

	if( str.IsEmpty() )	
		return;
	else
	{	i = str.GetLength();
		if( i>0 )
			if( str.GetAt(i-1)=='\n' )
				str = str.Left(i-1);
	}

	m_Infoline.GetWindowText(strBuffer);
	if ( !strBuffer.IsEmpty() )
		strBuffer += "\r\n";
	strBuffer += str;
	InfoLineCount++;
	if( InfoLineCount>=LINECOUNTMAX )
	{	char* ptr = strBuffer.LockBuffer();
		int j = 0;
		i = 0;
		while( i<20 && *ptr )
		{	if( *ptr++ =='\n' )
				i++;
			j++;
		}
		strBuffer.UnlockBuffer();
		if( j>=0 )
		{	InfoLineCount -= i;
			strBuffer = strBuffer.Right(strBuffer.GetLength()-j-1);
		}
	}
	m_Infoline.SetWindowText(strBuffer);
	m_Infoline.LineScroll(m_Infoline.GetLineCount(),0);
	ChangeStatusLight(true);
}

void CPannelBar::ChangeStatusLight(int Status)
{	UINT nIDLight;
 
	StatusLight.DeleteObject();

	switch( Status )
	{	case(0):	nIDLight = 	IDB_LIGHT_OFF;
					break;
		case(1):	if( IsPaused )
					{	CWnd *pFoc = GetFocus( );
						if( pFoc )
						{	if( !pFoc->IsKindOf(RUNTIME_CLASS(CRasView)) )
								nIDLight = IDB_LIGHT_STOP;
							else
								nIDLight = IDB_LIGHT_PAUSE;
						}
					} else if( FileDepth!=-1 )
					{	nIDLight = IDB_LIGHT_SCRIPT;
					} else
					{	nIDLight = IDB_LIGHT_ON;
					}
					break;
		case(-1):	nIDLight = 	IDB_LIGHT_STOP;
	}

	StatusLight.LoadBitmap(nIDLight);
	m_RasmolStatus.SetBitmap(StatusLight.operator HBITMAP());

	CRect rec;
	m_RasmolStatus.GetClientRect((LPRECT) rec) ;
	CDC* pDC = m_RasmolStatus.GetDC();
	pDC->BitBlt( 0, 0, rec.right, rec.bottom, NULL, 0, 0, SRCCOPY);
	m_RasmolStatus.ReleaseDC(pDC);
	m_RasmolStatus.Invalidate(true);
	m_RasmolStatus.UpdateWindow();
}

void CPannelBar::OnHScroll( UINT nSBCode, UINT nPos, CScrollBar* pScrollBar )
{	//improve control handling
	pScrollBar->Invalidate(true);
	pScrollBar->UpdateWindow();

	//remove redundant messages
	MSG msg;
	if( ::PeekMessage(&msg, m_hWnd, WM_LBUTTONUP, WM_LBUTTONUP, PM_NOREMOVE) )
		while(::PeekMessage(&msg, m_hWnd, WM_MOUSEMOVE, WM_MOUSEMOVE, PM_REMOVE));

	GETFRAME
	GETVIEW
	if( pView==NULL )
		return;

	if(	pScrollBar==&m_ScrollX ||
		pScrollBar==&m_ScrollY ||
		pScrollBar==&m_ScrollZ )
	{

	/*	Non correspondance between DialValues and scrollbars
	 *	DialValue	0	rotate	X (y screen values, inverted on IBMPC)
	 *	DialValue	1	rotate	Y
	 *	DialValue	2	rotate	Z
	 *	DialValue	3	zoom
	 *	DialValue	4	translate X
	 *	DialValue	5	translate Y
	 */
	
	if( m_spinStatus && (nSBCode == SB_LINEDOWN || nSBCode == SB_LINEUP) )	
	{	int s = pScrollBar->GetDlgCtrlID( ); 
		double t = (nSBCode == SB_LINEUP) ? -0.0005:0.0005;
		switch(s)
		{	case(IDC_ROTATEX):	SpinX += -t;break;
			case(IDC_ROTATEY):	SpinY += t;	break;
			case(IDC_ROTATEZ):	SpinZ += t;	break;
			default:            return;
		}
		return;
	}
	
	int pos = pScrollBar->GetScrollPos();

	switch( nSBCode )
	{	case(SB_LINEDOWN):  pos += 4;   break;
		case(SB_PAGEDOWN):  pos += 20;  break;
		case(SB_PAGEUP):    pos -= 20;  break;
		case(SB_LINEUP):    pos -= 4;   break;
		case(SB_THUMBTRACK):
		case(SB_THUMBPOSITION):
							if( pos == (int)nPos )
								return;
							pos = nPos;	break;
		case(SB_ENDSCROLL):
		case(SB_LEFT):	
		case(SB_RIGHT):		
		default:    //button up
					if( Shadow_Flag )
					{	Shadow_Flag = false;
						if( FontStroke_Flag )
							pView->ExecuteCommand( DisplayOff, "set shadow on\n");
						else
							pView->ExecuteCommand( DisplayOn, "set shadow on\n");
					}
					if( FontStroke_Flag )
					{	CString Command;
						Command.Format("set fontstroke %d\n",FontStroke_Flag);
						pView->ExecuteCommand( DisplayOn, Command);
						FontStroke_Flag = 0;
					}
					return;
	}

	if( pos>200 ) 
		pos -= 200;
	if( pos<0 ) 
		pos += 200; 

		pScrollBar->SetScrollPos( pos, true );

	if( Database )
	{	int s = pScrollBar->GetDlgCtrlID(); 
		int DialFlag;
		int Switch = m_modetranslate.GetCheck();
		switch( s )
		{	case(IDC_ROTATEX):	DialFlag = 0 + 4*Switch;	break;
			case(IDC_ROTATEY):	DialFlag = 1 + 4*Switch;	break;
			case(IDC_ROTATEZ):	DialFlag = 2 + Switch;		break;
		}
		switch( DialFlag )
		{	case(0):	DialValue[0] = 1 - pos/100.0;  
						ReDrawFlag |= RFRotateX;	
						break;
			case(1):	DialValue[1] = pos/100.0 - 1;
						ReDrawFlag |= RFRotateY;
						break;
			case(2):	DialValue[2] = 1 - pos/100.0; 
						ReDrawFlag |= RFRotateZ;	
						break;
			case(3):	DialValue[3] = pos/100.0 - 1; 
						ReDrawFlag |= RFZoom;		
						break;
			case(4):	DialValue[4] = pos/100.0 - 1; 
						ReDrawFlag |= RFTransX;	
						break;
			case(5):	DialValue[5] = 1 - pos/100.0;
						ReDrawFlag |= RFTransY;	
						break;
		}

		if( UseShadow )
		{	Shadow_Flag = true;
			pView->ExecuteCommand(DisplayOff, "set shadow off\n");
		}
		if( FontStroke != 0 )
		{   FontStroke_Flag = FontStroke;
			pView->ExecuteCommand( DisplayOff, "set fontstroke 0\n");
		}

		RefreshView();
	}
	return;
	} else if( &m_Slab==(CSliderCtrl *)pScrollBar )
	{ 
		Real pos = m_Slab.GetPos();
		if( pos>200 ) 
			pos = 200;
		if( pos<0 ) 
			pos = 0; 

		if( Database )
		{	if( pos==0 )
				pView->ExecuteCommand(DisplayOn, "slab off\n");
			else
			{	CString Command;
				Command.Format("slab %.1f\n",(100 - pos/2));
				pView->ExecuteCommand(DisplayOn, Command);
			}
		}
	} else if( &m_Depth==(CSliderCtrl *)pScrollBar )
	{ 
		Real pos = m_Depth.GetPos();
		if( pos>200 ) 
			pos = 200;
		if( pos<0 ) 
			pos = 0; 

		if( Database )
		{	if( pos==200 )
				pView->ExecuteCommand(DisplayOn,"depth off\n");
			else
			{	CString Command;
				Command.Format("depth %.1f\n",(100 - pos/2));
				pView->ExecuteCommand(DisplayOn,Command);
			}
		}

	} else if( &m_Light	== (CSliderCtrl *) pScrollBar )
	{	int pos = m_Light.GetPos();
		if( pos>100 ) 
			pos = 100;
		if( pos<0 ) 
			pos = 0; 
		if(Database)
		{	if( Ambient!=pos/100.0 )
			{	Ambient = pos/100.0;
				ReDrawFlag |= RFColour;
				RefreshView();
			}		
		}
	} else if( &m_Specular==(CSpinButtonCtrl *)pScrollBar )
	{	int pos = LOWORD(m_Specular.GetPos());
		if( pos>14 ) 
			pos = 14;
		if( pos<0 ) 
			pos = 0; 
		if( Database )
		{	int expos = 14;
			if( !FakeSpecular )			expos = 14;
			else if( SpecPower<10 )		expos = SpecPower;
			else if( SpecPower<20 )		expos = 10;
			else if( SpecPower<40 )		expos = 11;
			else if( SpecPower<80 )		expos = 12;
			else if( SpecPower<100)		expos = 13;

			if( expos!=pos )		
			{	CString buddy;
				FakeSpecular = true;
				switch(pos)
				{	case(11):	SpecPower = 20;
								break;
					case(12):	SpecPower = 40;
								break;
					case(13):	SpecPower = 80;
								break;
					case(14):	SpecPower = 100;
								FakeSpecular = false;
								break;
					default:	SpecPower = pos;
				}
				buddy.Format("%d",SpecPower);
				if( !FakeSpecular )
					buddy = "off";
				m_BuddySpec.SetWindowText(buddy);
				m_BuddySpec.UpdateWindow();
				ReDrawFlag |= RFColour;
				RefreshView();
			}	
		}
	} else if( &m_Shade==(CSpinButtonCtrl *)pScrollBar )
	{	int pos = LOWORD(m_Shade.GetPos());
		if( pos>40 ) 
			pos = 40;
		if( pos<0 ) 
			pos = 0; 
		if( Database )
		{	if( pos!=ShadePower )
			{	ShadePower = pos-20;
				CString buddy;
				buddy.Format("%d",ShadePower);
				m_BuddyShade.SetWindowText(buddy);
				m_BuddyShade.UpdateWindow();
				ReDrawFlag |= RFColour;
				RefreshView();
			}
		}
	}			
}

void CPannelBar::OnTimer(UINT nIDEvent) 
{	if( nIDEvent == m_PannelTimerID_1 )
	{	m_InfoTimerStatus = true;
		if( KillTimer(m_PannelTimerID_1) )
			m_PannelTimerID_1 = 0;
		infopointeroff.DeleteObject();
		infopointeroff.LoadBitmap(IDB_INFOPOINTEROFF);
		infopointer.SetBitmap(infopointeroff.operator HBITMAP());
		return;
	}

	if( nIDEvent == m_PannelTimerID_0 )
	{	KillTimerPannel();
		if( !Database )
			ResetSpinMode();

		GETFRAME
		GETVIEW

		/*Setting of dial variation to drawing time
		 * 0.001 dial unit per 25 ms
		 */
		if( pView->m_drawingtime>=2 )
		{	DialValue[0] += (SpinX*pView->m_drawingtime)/25;
			DialValue[1] += (SpinY*pView->m_drawingtime)/25;
			DialValue[2] -= (SpinZ*pView->m_drawingtime)/25;
		} else 
		{	DialValue[0] += SpinX;
			DialValue[1] += SpinY;
			DialValue[2] -= SpinZ;
		}

		for( int i = 0; i<3;i++ )
		{	if(DialValue[i] > 1.0) DialValue[i] -= 2.0;
			if(DialValue[i] < -1.0) DialValue[i] += 2.0;
		}

		ReDrawFlag |= RFRotateX | RFRotateY | RFRotateZ;
		RefreshView();		
		UpdateScrollBars();
		SetTimerPannel();
	}
}

void CPannelBar::OnRotateButton()		{	UpdateScrollBars();	}
void CPannelBar::OnTranszoomButton()	{	UpdateScrollBars();	}

void CPannelBar::SetTimerPannel()
{	static BOOL OkTimer = true;
	if( OkTimer&&m_spinStatus )
	{	m_PannelTimerID_0 = SetTimer(IDT_TIMER_0, 1, NULL);
		if( m_PannelTimerID_0==0 )
		{	MessageBox("Unable to obtain a timer for spinmode!",
			 NULL, MB_OK|MB_SYSTEMMODAL);
			OkTimer =  false;
		}
	}
}

void CPannelBar::KillTimerPannel()
{	if( m_spinStatus )
		if( KillTimer(m_PannelTimerID_0) )
			m_PannelTimerID_0 = 0;
}

void CPannelBar::ResetSpinMode()
{	KillTimerPannel();	
	m_spinStatus = false;
	m_modespin.SetCheck(false);
	SpinX = SpinY = SpinZ = 0;
}

void CPannelBar::OnSpinButton() 
{	if( !Database )
	{	m_modespin.SetCheck(false);
		return;
	}

	if( !m_spinStatus )
	{	m_spinStatus = true;
		m_modespin.SetCheck(true);
		if( (SpinX==0) && (SpinY==0) && (SpinZ==0) )
			SpinY = 0.001;
		SetTimerPannel();
	} else
		ResetSpinMode();
}

void CPannelBar::OnResetButton() 
{	m_reset.SetCheck(false);
	GETFRAME
	GETVIEW
	if( pView!=NULL ) 
	{	pView->ExecuteCommand(DisplayOff, "slab off\n");
		pView->ExecuteCommand(DisplayOff, "depth off\n");
		pView->ExecuteCommand(DisplayOn, "reset\n");
	}
	::SetPickMode( PickIdent );
	ResetSpinMode();
}

void CPannelBar::OnUpdateSlabSlid(CCmdUI *pCmdUI)
{	m_Slab.SetRange(0, 200, false);	}
void CPannelBar::OnUpdateDepthSlid(CCmdUI *pCmdUI)
{	m_Depth.SetRange(0, 200, false);	}
void CPannelBar::OnUpdateLightSlid(CCmdUI *pCmdUI)
{	m_Light.SetRange(0, 100, false);	}

void CPannelBar::OnCloseButton()
{	GETFRAME
	pFrame->DesactivateCommandBar();

	//Give back focus to view during paused script
	GETVIEW
	if( pView != NULL )
		pView->SetFocus();
}

void CPannelBar::OnUpdateCloseButton(CCmdUI *pCmdUI)
{	pCmdUI->Enable(true);	}
 
void CPannelBar::RefreshView()
{	GETFRAME
	GETVIEW
	if( pView==NULL )
		return;
	/*RefreshView();*/
	pView->Invalidate(true);
}

void CPannelBar::UpdateScrollBars()
{	int pos;

	if( m_moderotate.GetCheck() )
	{   pos = 100 + (int)(100.0*DialValue[0]);
		m_ScrollX.SetScrollPos( 200 - pos, true );

		pos = 100 + (int)(100.0*DialValue[1]);
		m_ScrollY.SetScrollPos( pos, true );
	
	    pos = 100 - (int)(100.0*DialValue[2]);
		m_ScrollZ.SetScrollPos( pos, true );
	} else
	{   pos = 100 + (int)(100.0*DialValue[4]);
		m_ScrollX.SetScrollPos( pos, true );
	
	    pos = 100 + (int)(100.0*DialValue[5]);
		m_ScrollY.SetScrollPos( 200 - pos, true );
	
	    pos = 100 + (int)(100.0*DialValue[3]);
		m_ScrollZ.SetScrollPos( pos, true );
	}
}

void CPannelBar::UpdatePannel()
{	double pos;

	UpdateScrollBars();

	if( UseSlabPlane )
		pos = 100 - 100*DialValue[7];
	else pos = 0;
	m_Slab.SetPos((int)pos);

	if( UseDepthPlane )
		pos = 100 - 100*DialValue[8];
	else pos = 200;
	m_Depth.SetPos((int)pos);

	m_Light.SetPos((int)(100*Ambient));

	CString buddy;
	pos = 14;
	if( FakeSpecular )
	{	if( SpecPower<10 )
		{	pos = SpecPower;
		} else if( SpecPower<20 )
		{	pos = 10;
		} else if( SpecPower<40 )
		{	pos = 11;
		} else if( SpecPower<80 )
		{	pos = 12;
		} else if( SpecPower<100 )
		{	pos = 13;
		}
	}
	buddy.Format("%d",SpecPower);
	if( !FakeSpecular )
		buddy = "off";
	m_BuddySpec.SetWindowText(buddy);
	m_Specular.SetPos((int)pos);

	buddy.Format("%d",ShadePower);
	m_BuddyShade.SetWindowText(buddy);
	m_Shade.SetPos(ShadePower+20);
}

