/***************************************************************************
 *                                                                         *
 *                               RasTop 1.3.1                              *
 *                 Molecular Graphics Visualisation Tool                   *
 *                          by Philippe Valadon                            *
 *                              21 Mars 2001                               *
 *                                                                         *
 *                                *******                                  *
 *                                                                         *
 *                   Based on RasMol 2.6 by Roger Sayle                    *
 * Biomolecular Structures Group, Glaxo Wellcome Research & Development,   *
 *                      Stevenage, Hertfordshire, UK                       *
 *         Version 2.6, August 1995, Version 2.6.4, December 1998          *
 *                   Copyright (C) Roger Sayle 1992-1999                   *
 *                                                                         *
 *                  and Based on Mods by Arne Mueller                      *
 *                      Version 2.6x1, May 1998                            *
 *                   Copyright (C) Arne Mueller 1998                       *
 *                                                                         *
 *    and Based on Version 2.7.0, 2.7.1 Mods by Herbert J. Bernstein       *
 *           Bernstein + Sons, P.O. Box 177, Bellport, NY, USA             *
 *                      yaya@bernstein-plus-sons.com                       *
 *                    2.7.0 March 1999, 2.7.1 June 1999                    *
 *              Copyright (C) Herbert J. Bernstein 1998-1999               *
 *                                                                         *
 *                RasTop Version 1.3.x by Philippe Valadon                 *
 *                 P.O. Box 13014, La Jolla, CA 92039, USA,                *
 *                         valadon@ix.netcom.com                           *
 *                Copyright (C) Philippe Valadon 2000-2001                 *
 *                                                                         *
 *                                *******                                  *
 *                                                                         *
 * As written by Herbert Bernstein in Rasmol Version 2.7.1:                *
 *                                                                         *
 * Please read the file NOTICE for important notices which apply to this   *
 * package. If you are not going to make changes to Rasmol, you are not    *
 * only permitted to freely make copies and distribute them, you are       *
 * encouraged to do so, provided you do the following:                     *
 *   * 1. Either include the complete documentation, especially the file   *
 *     NOTICE, with what you distribute or provide a clear indication      *
 *     where people can get a copy of the documentation; and               *
 *   * 2. Please give credit where credit is due citing the version and    *
 *     original authors properly; and                                      *
 *   * 3. Please do not give anyone the impression that the original       *
 *     authors are providing a warranty of any kind.                       *
 *                                                                         *
 * If you would like to use major pieces of Rasmol in some other program,  *
 * make modifications to Rasmol, or in some other way make what a lawyer   *
 * would call a "derived work", you are not only permitted to do so, you   *
 * are encouraged to do so. In addition to the things we discussed above,  *
 * please do the following:                                                *
 *   * 4. Please explain in your documentation how what you did differs    *
 *     from this version of Rasmol; and                                    *
 *   * 5. Please make your modified source code available.                 *
 *                                                                         *
 * This version of Rasmol is not in the public domain, but it is given     *
 * freely to the community in the hopes of advancing science. If you make  *
 * changes, please make them in a responsible manner, and please offer us  *
 * the opportunity to include those changes in future versions of RasTop.  *
 ***************************************************************************/

/////////////////////////////////////////////////////////////////////////////
// rasdoc.h
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_RASDOC_H_INCLUDED_)
#define AFX_RASDOC_H_INCLUDED_

#include <afxtempl.h>
#include "datadisplay.h"

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

char *RegisterFormat( char *buffer, char *desc, char *ext );
BOOL LoadInputFile(int format);

class CRasDoc : public CDocument
{
protected:
	CRasDoc();
	DECLARE_DYNCREATE(CRasDoc)

// Attributes
public:

// Operations
public:

// Overrides
	//{{AFX_VIRTUAL(CRasDoc)
	public:
	virtual BOOL OnNewDocument();
	virtual BOOL OnSaveDocument(LPCTSTR lpszPathName);
	virtual BOOL OnOpenDocument(LPCTSTR lpszPathName);
	virtual void OnCloseDocument();
	protected:
	virtual BOOL SaveModified();
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CRasDoc();

#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

public:
	static int m_MoleculeCount;		//Count number of opened molecules
	CString m_FileData;				//Molecular data

	CData m_DataDisplay;			//Data exchange with rasmol
	int ViewActivationFlag;			//Overcome duplicat of activation/desactivation

	HBITMAP m_PixMap;

	void CheckActivationState();
	void FixRange();
	void ManageDisplay(BOOL ActivationFlag);
	int ManageOpening(LPCTSTR lpszPathName);
	int ProcessFileData( );
	void SetDocStatus( BOOL Status );

protected:
	//{{AFX_MSG(CRasDoc)
	afx_msg void OnSaveMolecularfile();
	afx_msg void OnUpdateSaveMolecularfile(CCmdUI* pCmdUI);
	afx_msg void OnSaveImage();
	afx_msg void OnUpdateSaveImage(CCmdUI* pCmdUI);
	afx_msg void OnSaveScript();
	afx_msg void OnUpdateSaveScript(CCmdUI* pCmdUI);
	afx_msg void OnFileLoadRasmolscript();
	afx_msg void OnLoadMolecularfile();
	afx_msg void OnFileOpenFromDoc();
	afx_msg void OnUpdateFileSave(CCmdUI* pCmdUI);
	afx_msg void OnUpdateFileSaveAs(CCmdUI* pCmdUI);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

//{{AFX_INSERT_LOCATION}}

#endif // !defined(AFX_RASDOC_H_INCLUDED_)
