/***************************************************************************
 *                                                                         *
 *                               RasTop 1.3.1                              *
 *                 Molecular Graphics Visualisation Tool                   *
 *                          by Philippe Valadon                            *
 *                              21 Mars 2001                               *
 *                                                                         *
 *                                *******                                  *
 *                                                                         *
 *                   Based on RasMol 2.6 by Roger Sayle                    *
 * Biomolecular Structures Group, Glaxo Wellcome Research & Development,   *
 *                      Stevenage, Hertfordshire, UK                       *
 *         Version 2.6, August 1995, Version 2.6.4, December 1998          *
 *                   Copyright (C) Roger Sayle 1992-1999                   *
 *                                                                         *
 *                  and Based on Mods by Arne Mueller                      *
 *                      Version 2.6x1, May 1998                            *
 *                   Copyright (C) Arne Mueller 1998                       *
 *                                                                         *
 *    and Based on Version 2.7.0, 2.7.1 Mods by Herbert J. Bernstein       *
 *           Bernstein + Sons, P.O. Box 177, Bellport, NY, USA             *
 *                      yaya@bernstein-plus-sons.com                       *
 *                    2.7.0 March 1999, 2.7.1 June 1999                    *
 *              Copyright (C) Herbert J. Bernstein 1998-1999               *
 *                                                                         *
 *                RasTop Version 1.3.x by Philippe Valadon                 *
 *                 P.O. Box 13014, La Jolla, CA 92039, USA,                *
 *                         valadon@ix.netcom.com                           *
 *                Copyright (C) Philippe Valadon 2000-2001                 *
 *                                                                         *
 *                                *******                                  *
 *                                                                         *
 * As written by Herbert Bernstein in Rasmol Version 2.7.1:                *
 *                                                                         *
 * Please read the file NOTICE for important notices which apply to this   *
 * package. If you are not going to make changes to Rasmol, you are not    *
 * only permitted to freely make copies and distribute them, you are       *
 * encouraged to do so, provided you do the following:                     *
 *   * 1. Either include the complete documentation, especially the file   *
 *     NOTICE, with what you distribute or provide a clear indication      *
 *     where people can get a copy of the documentation; and               *
 *   * 2. Please give credit where credit is due citing the version and    *
 *     original authors properly; and                                      *
 *   * 3. Please do not give anyone the impression that the original       *
 *     authors are providing a warranty of any kind.                       *
 *                                                                         *
 * If you would like to use major pieces of Rasmol in some other program,  *
 * make modifications to Rasmol, or in some other way make what a lawyer   *
 * would call a "derived work", you are not only permitted to do so, you   *
 * are encouraged to do so. In addition to the things we discussed above,  *
 * please do the following:                                                *
 *   * 4. Please explain in your documentation how what you did differs    *
 *     from this version of Rasmol; and                                    *
 *   * 5. Please make your modified source code available.                 *
 *                                                                         *
 * This version of Rasmol is not in the public domain, but it is given     *
 * freely to the community in the hopes of advancing science. If you make  *
 * changes, please make them in a responsible manner, and please offer us  *
 * the opportunity to include those changes in future versions of RasTop.  *
 ***************************************************************************/

///////////////////////////////////////////////////////////////////////////////
//SelectBar

#define COMBOBOX_INDEX	6
#define COMBOBOX_WIDTH	115
#define COMBOBOX_HEIGHT 340

#define SET_SEPARATOR	"--------------------"

#include "stdafx.h"
#include "RasTop.h"
#include "rasdoc.h"
#include "graphics.h"
#include "rasview.h"
#include "mainfrm.h"
#include "selectbar.h"

static UINT BASED_CODE SelectButtons[] =
{	ID_SELECT_ALL,
	ID_SELECT_PREVIOUS,
	ID_SELECT_INVERSE,
	ID_SELECT_EXPRESSION,
	ID_SELECT_SHOW,
		ID_SEPARATOR,
		ID_SEPARATOR,
		ID_SEPARATOR,
	ID_SELECTION_NEW,
	ID_SELECTION_ADD,
	ID_SELECTION_REMOVE,
	ID_SELECTION_EXTRACT,
	ID_SELECTION_EXCLUDE,
		ID_SEPARATOR,
	ID_PICK_ATOM,
	ID_PICK_GROUP,
	ID_PICK_CHAIN,
	ID_PICK_BOND,
	ID_PICK_CENTRE,
	ID_PICK_LABEL,
	ID_PICK_MONITOR,
	ID_PICK_DISTANCE,
	ID_PICK_ANGLE,
	ID_PICK_TORSION,
};

CSelectBar::CSelectBar()	{}

CSelectBar::~CSelectBar()	{}

BOOL CSelectBar::Init(CWnd* pParentWnd)
{
	DWORD dwStyle = WS_CHILD | WS_VISIBLE | CBRS_TOP | CBRS_SIZE_FIXED
		| CBRS_TOOLTIPS | CBRS_FLYBY;
	if( !Create(pParentWnd, dwStyle, IDW_SELECTBAR) )
		return false;
	
	if( !LoadBitmap(IDR_SELECTIONBAR) )
		return false;

	if( !SetButtons(SelectButtons, sizeof(SelectButtons)/sizeof(UINT)) )
		return false;

	CRect rect(-COMBOBOX_WIDTH, -COMBOBOX_HEIGHT, 0, 0);
	if( !m_SearchBox.Create(WS_CHILD | CBS_DROPDOWNLIST | 
		CBS_AUTOHSCROLL | WS_VSCROLL | CBS_HASSTRINGS, rect, this,
		IDC_SEARCHBOX) )
		return false;
	
	//Create font
	HFONT hFont = (HFONT)GetStockObject(DEFAULT_GUI_FONT);
	if (hFont == NULL)
		hFont = (HFONT)GetStockObject(ANSI_VAR_FONT);
  	m_SearchBox.SendMessage(WM_SETFONT, (WPARAM)hFont);

	SetButtonInfo(COMBOBOX_INDEX, IDC_SEARCHBOX, 
		TBBS_SEPARATOR, COMBOBOX_WIDTH);

	if( m_SearchBox.m_hWnd!=NULL )
	{
		CRect rect;
		GetItemRect(COMBOBOX_INDEX, rect);
		
		m_SearchBox.SetWindowPos(NULL, rect.left, rect.top, 0, 0, 
			SWP_NOZORDER|SWP_NOACTIVATE|SWP_NOSIZE|SWP_NOCOPYBITS);
		m_SearchBox.ShowWindow(SW_SHOW);
	}

	//Fill the box with predefined sets
	m_SearchBox.InsertString( -1, "no selection");
	m_SearchBox.InsertString( -1, "at"			);
	m_SearchBox.InsertString( -1, "acidic"		);
	m_SearchBox.InsertString( -1, "acyclic"		);
	m_SearchBox.InsertString( -1, "aliphatic"	);
	m_SearchBox.InsertString( -1, "alpha"		);
	m_SearchBox.InsertString( -1, "amino"		);
	m_SearchBox.InsertString( -1, "aromatic"	);
	m_SearchBox.InsertString( -1, "backbone"	);
	m_SearchBox.InsertString( -1, "basic"		);
	m_SearchBox.InsertString( -1, "bonded"		);
	m_SearchBox.InsertString( -1, "buried"		);
	m_SearchBox.InsertString( -1, "cg"			);
	m_SearchBox.InsertString( -1, "charged"		);
	m_SearchBox.InsertString( -1, "cisbonded"	);
	m_SearchBox.InsertString( -1, "cyclic"		);
	m_SearchBox.InsertString( -1, "cystine"		);
	m_SearchBox.InsertString( -1, "DNA"			);
	m_SearchBox.InsertString( -1, "helix"		);
	m_SearchBox.InsertString( -1, "hetero"		);
	m_SearchBox.InsertString( -1, "hydrogen"	);
	m_SearchBox.InsertString( -1, "hydrophobic"	);
	m_SearchBox.InsertString( -1, "ions"		);
	m_SearchBox.InsertString( -1, "large"		);
	m_SearchBox.InsertString( -1, "ligand"		);
	m_SearchBox.InsertString( -1, "medium"		);
	m_SearchBox.InsertString( -1, "mainchain"	);
	m_SearchBox.InsertString( -1, "neutral"		);
	m_SearchBox.InsertString( -1, "nucleic"		);
	m_SearchBox.InsertString( -1, "polar"		);
	m_SearchBox.InsertString( -1, "protein"		);
	m_SearchBox.InsertString( -1, "purine"		);
	m_SearchBox.InsertString( -1, "pyrimidine"	);
	m_SearchBox.InsertString( -1, "RNA"			);
	m_SearchBox.InsertString( -1, "sheet"		);
	m_SearchBox.InsertString( -1, "sidechain"	);
	m_SearchBox.InsertString( -1, "small"		);
	m_SearchBox.InsertString( -1, "solvent"		);
	m_SearchBox.InsertString( -1, "surface"		);
	m_SearchBox.InsertString( -1, "turn"		);
	m_SearchBox.InsertString( -1, "water"		);
	m_SearchBox.InsertString( -1, SET_SEPARATOR);
	m_SearchBox.SetCurSel( 0 );
	return true;
}

BEGIN_MESSAGE_MAP(CSelectBar, CToolBar)
	//{{AFX_MSG_MAP(CSelectBar)
	ON_WM_WINDOWPOSCHANGED()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_NCLBUTTONDBLCLK()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

void CSelectBar::OnLButtonDblClk(UINT nFlags, CPoint point) 
{	if( !IsFloating() )	
		CToolBar::OnLButtonDblClk(nFlags, point);
}

void CSelectBar::OnNcLButtonDblClk(UINT nHitTest, CPoint point) 
{	if( !IsFloating() )	
		CToolBar::OnNcLButtonDblClk(nHitTest, point);
}

void CSelectBar::SetBarPalette(BOOL bDraw)
{
	
	//Only one passage
	static BOOL bOnce = false;
		
	//Except redrawing floating bar from saved position
	if( !IsFloating( )||!bDraw )
		if( bOnce==bDraw) 
			return;

	bOnce = bDraw;

	int nCount = GetToolBarCtrl().GetButtonCount();

	for( int i = 0; i < nCount; i++ )
	{	
		UINT nStyle = GetButtonStyle(i);
		if( i==5||i==7||i==13||i==18 )
		{	if( bDraw )
				nStyle |= TBBS_WRAPPED;
			else
				nStyle &= ~TBBS_WRAPPED;
		}
		else
			nStyle &= ~TBBS_WRAPPED;
    	SetButtonStyle(i, nStyle);
	}
	
	if( m_SearchBox.m_hWnd != NULL )
	{
		CRect rect;
		GetItemRect(COMBOBOX_INDEX, rect);
		
		m_SearchBox.SetWindowPos(NULL, rect.left, rect.top, 0, 0, 
			SWP_NOZORDER|SWP_NOACTIVATE|SWP_NOSIZE|SWP_NOCOPYBITS);
		m_SearchBox.ShowWindow(SW_SHOW);
	}

	if( !IsFloating( ) && bDraw )
	{	CPoint point(550,150);
		GetParentFrame()->FloatControlBar( this, point , CBRS_ALIGN_TOP );
	}
	if( IsFloating( ) && !bDraw )
	{	GetParentFrame()->DockControlBar(this, AFX_IDW_DOCKBAR_TOP, NULL );
	}

	//Top docking
	if( !IsFloating( ) && !bDraw )
		((CMainFrame*)AfxGetApp()->m_pMainWnd)->DockSelectBar();

	if( bDraw )
		SetWindowText(_T("Selection"));

	Invalidate();
	GetParentFrame()->RecalcLayout();
}

void CSelectBar::OnWindowPosChanged(WINDOWPOS FAR* lpwndpos) 
{
	CToolBar::OnWindowPosChanged(lpwndpos);

	if( IsFloating( ) )
		SetBarPalette(true);
	else
		SetBarPalette(false);
}

void CSelectBar::AddKeyword(CString & Key)
{	if( m_SearchBox.FindString(-1, Key)==CB_ERR )
		m_SearchBox.InsertString(-1, Key);
}

void CSelectBar::ResetBox(BOOL Status)
{	int i,j;
	int nItem = m_SearchBox.GetCount( );
	CString ItemText;

	for( i=0;i<nItem;i++ )
	{	m_SearchBox.GetLBText(i,ItemText);
		if( ItemText==SET_SEPARATOR )
			break;
	}

	//delete extrasets anyway
	for( j=i+1;j<nItem;j++ )
		m_SearchBox.DeleteString(i+1);

	//Add defined atom sets
	if( Status && Database )
	{	SymEntry  **prev, *ptr;
		SymEntry __far *cur;
		SymEntry __far *temp;
		int test;
		CString szTemp;

		//Check envelop exists
		test = true;
		prev = &SymbolTable;
		while( *prev )
		{   ptr = *prev;
		    test = _stricmp("envelop",ptr->ident);
		    if( !test ) break;
		    prev = (test<0)? &(ptr->lft) : &(ptr->rgt);
		}
		if( test )
		{	sprintf(CurLine,"define envelop\n");
			::ExecuteCommand();	
		}
	
		//Add sets to box
		cur = SymbolTable;
		if( cur )
		{	while( cur->lft )
				cur = cur->lft;	

			while( cur )
			{   szTemp = cur->ident;
				szTemp.MakeLower();
				if( szTemp.Find("rasmol")==-1 )
					AddKeyword(szTemp);
				prev = &SymbolTable;
				temp = *prev;
				while( temp->rgt )
					temp = temp->rgt;	
				while( *prev )
				{   ptr = *prev;
					i = strcmp(temp->ident,ptr->ident);
				    j = strcmp(cur->ident,ptr->ident);
					if( i>0&&j<0 ) temp = ptr;
				    prev = (j<0)? &(ptr->lft) : &(ptr->rgt);
				}
				if( cur==temp )
					cur = NULL;
				else
					cur = temp;
			}
		}
	}

	m_SearchBox.SetCurSel( 0 );
}


/////////////////////////////////////////////////////////////////////////////
// CSelectBox

CSelectBox::CSelectBox()	{}

CSelectBox::~CSelectBox()	{}

BEGIN_MESSAGE_MAP(CSelectBox, CComboBox)
	//{{AFX_MSG_MAP(CSelectBox)
	ON_CONTROL_REFLECT(CBN_SELCHANGE, OnSelchange)
	ON_WM_WINDOWPOSCHANGING()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


void CSelectBox::OnSelchange() 
{	CString CurSelText;
	int nSel = GetCurSel( );
	GETFRAME
	GetLBText( nSel, CurSelText );
	if( nSel == CB_ERR || nSel == 0 || CurSelText==SET_SEPARATOR )
	{	pFrame->m_SelectString = "";
		SetCurSel( 0 );
	}
	else
		pFrame->m_SelectString = CurSelText;
}

void CSelectBox::OnWindowPosChanging(WINDOWPOS FAR* lpwndpos) 
{	//totally empirical formulation to prevent some erratic drawing on dblclik
	//but does not work on NT
	if( lpwndpos->x!=0 && lpwndpos->x!=123 )
	{	lpwndpos->x = 123;
		lpwndpos->y = 2;
	}
		

	CComboBox::OnWindowPosChanging(lpwndpos);
	
}
